@@ -29,7 +29,6 @@
 #include <util/misc.h>
 
 #include "jsapi-util.h"
-#include "context-jsapi.h"
 #include "compat.h"
 #include "jsapi-private.h"
 
@@ -41,8 +40,21 @@ gjs_util_error_quark (void)
     return g_quark_from_static_string ("gjs-util-error-quark");
 }
 
+typedef struct {
+    JSContext *context;
+    int depth;
+} ContextFrame;
+
 typedef struct {
     GHashTable *dynamic_classes;
+
+    JSObject *import_global;
+
+    JSContext *default_context;
+
+    /* In a thread-safe future we'd keep this in per-thread data */
+    ContextFrame current_frame;
+    GSList *context_stack;
 } RuntimeData;
 
 typedef struct {
@@ -50,20 +62,176 @@ typedef struct {
     JSClass *static_class;
 } DynamicJSClass;
 
-void*
-gjs_runtime_get_data(JSRuntime      *runtime,
-                     const char     *name)
+static RuntimeData* get_data_from_runtime(JSRuntime *runtime);
+static RuntimeData* get_data_from_context(JSContext *context);
+
+/**
+ * gjs_get_import_global:
+ * @context: a #JSContext
+ *
+ * Gets the "import global" for the context's runtime. The import
+ * global object is the global object for the default context. It is used
+ * as the root object for the scope of modules loaded by GJS in this
+ * runtime, and should also be used as the globals 'obj' argument passed
+ * to JS_InitClass() and the parent argument passed to JS_ConstructObject()
+ * when creating a native classes that are shared between all contexts using
+ * the runtime. (The standard JS classes are not shared, but we share
+ * classes such as GObject proxy classes since objects of these classes can
+ * easily migrate between contexts and having different classes depending
+ * on the context where they were first accessed would be confusing.)
+ *
+ * Return value: the "import global" for the context's
+ *  runtime. Will never return %NULL while GJS has an active context
+ *  for the runtime.
+ */
+JSObject*
+gjs_get_import_global(JSContext *context)
 {
-    return g_dataset_get_data(runtime, name);
+    JSRuntime *runtime = JS_GetRuntime(context);
+    RuntimeData *rd;
+
+    rd = get_data_from_runtime(runtime);
+
+    return rd->import_global;
 }
 
+/**
+ * gjs_runtime_push_context:
+ * @runtime: a #JSRuntime
+ * @context: a #JSRuntime
+ *
+ * Make @context the currently active context for @runtime.
+ * A stack is maintained, although switching between different contexts
+ * in a nested fashion in the same thread ecan trigger misbehavior in
+ * Spidermonkey, so is not recommended. This does not call JS_BeginRequest();
+ * the caller needs to do it themselves.
+ *
+ * Should be called when calling from Javascript into native code that
+ * could result in callbacks back to Javascript. The context stack allows
+ * the callbacks to find the right context to use via gjs_get_current_context().
+ *
+ * When GJS is made threadsafe, this needs to maintain a per-thread stack
+ * rather than a global stack.
+ */
 void
-gjs_runtime_set_data(JSRuntime      *runtime,
-                     const char     *name,
-                     void           *data,
-                     GDestroyNotify  dnotify)
+gjs_runtime_push_context(JSRuntime *runtime,
+                         JSContext *context)
 {
-    g_dataset_set_data_full(runtime, name, data, dnotify);
+    RuntimeData *rd;
+
+    rd = get_data_from_runtime(runtime);
+
+    if (context == rd->current_frame.context) {
+        rd->current_frame.depth++;
+    } else {
+        rd->context_stack = g_slist_prepend(rd->context_stack,
+                                            g_slice_dup(ContextFrame, &rd->current_frame));
+        rd->current_frame.context = context;
+        rd->current_frame.depth = 0;
+    }
+}
+
+/**
+ * gjs_runtime_pop_context:
+ * @runtime: a #JSRuntime
+ *
+ * Pops a context pushed onto the stack of active contexts by
+ * gjs_runtime_push_context().
+ */
+void
+gjs_runtime_pop_context(JSRuntime *runtime)
+{
+    RuntimeData *rd;
+
+    rd = get_data_from_runtime(runtime);
+
+    if (rd->current_frame.depth == 0) {
+        if (rd->context_stack == NULL)
+            gjs_fatal("gjs_runtime_pop_context() called more times than gjs_runtime_push_context()");
+
+        rd->current_frame = *(ContextFrame *)rd->context_stack->data;
+        g_slice_free(ContextFrame, rd->context_stack->data);
+        rd->context_stack = g_slist_delete_link(rd->context_stack, rd->context_stack);
+    } else {
+        rd->current_frame.depth--;
+    }
+}
+
+/**
+ * gjs_runtime_set_default_context:
+ * @runtime: a #JSRuntime
+ * @context: a #JSContext
+ *
+ * Makes @context the default context for @runtime. The default context is the
+ * context used for executing callbacks when no other context is active.
+ * This generally should only be called by GJS - GJS sets the default context
+ * when #GjsContext creates a runtime, and subsequent calls to this function
+ * will produce an error.
+ */
+void
+gjs_runtime_set_default_context(JSRuntime *runtime,
+                                JSContext *context)
+{
+    RuntimeData *rd;
+
+    rd = get_data_from_runtime(runtime);
+
+    if (rd->context_stack != NULL || rd->current_frame.depth != 0)
+        gjs_fatal("gjs_runtime_set_default_context() called during gjs_push_context()");
+
+    if (context != NULL) {
+        if (rd->default_context != NULL)
+            gjs_fatal("gjs_runtime_set_default_context() called twice on the same JSRuntime");
+        rd->default_context = context;
+        rd->current_frame.context = context;
+        rd->import_global = JS_GetGlobalObject(rd->default_context);
+    } else {
+        rd->default_context = NULL;
+        rd->current_frame.context = NULL;
+        rd->import_global = NULL;
+    }
+}
+
+/**
+ * gjs_runtime_get_default_context:
+ * @runtime: a #JSRuntime
+ *
+ * Gets the default context for @runtime. Generally you should use
+ * gjs_runtime_get_current_context() instead.
+ *
+ * Return value: the default context, or %NULL if GJS hasn't been initialized
+ *  for the runtime or is being shut down.
+ */
+JSContext *
+gjs_runtime_get_default_context(JSRuntime *runtime)
+{
+    RuntimeData *rd;
+
+    rd = get_data_from_runtime(runtime);
+
+    return rd->default_context;
+}
+
+/**
+ * gjs_runtime_get_current_context:
+ * @runtime: a #JSRuntime
+ *
+ * Gets the right context to use for code that doesn't already have a JSContext
+ * passed to it, like a callback from native code. If a context is currently
+ * active (see gjs_push_context()), uses that, otherwise uses the default
+ * context for the runtime.
+ *
+ * Return value: the current context, or %NULL if GJS hasn't been initialized
+ *  for the runtime or is being shut down.
+ */
+JSContext *
+gjs_runtime_get_current_context(JSRuntime *runtime)
+{
+    RuntimeData *rd;
+
+    rd = get_data_from_runtime(runtime);
+
+    return rd->current_frame.context;
 }
 
 static JSClass global_class = {
@@ -86,7 +254,7 @@ gboolean
 gjs_init_context_standard (JSContext       *context)
 {
     JSObject *global;
-#ifdef HAVE_JS_NEWGLOBALOBJECT
+#ifdef HAVE_MOZJS_2
     global = JS_NewGlobalObject(context, &global_class);
     if (global == NULL)
         return FALSE;
@@ -101,120 +269,67 @@ gjs_init_context_standard (JSContext       *context)
     return TRUE;
 }
 
-
-/* The "load context" is the one we use for loading
- * modules and initializing classes.
- */
-JSContext*
-gjs_runtime_get_load_context(JSRuntime *runtime)
-{
-    GjsContext *context;
-
-    context = gjs_runtime_get_data(runtime, "gjs-load-context");
-    if (context == NULL) {
-        gjs_debug(GJS_DEBUG_CONTEXT,
-                  "Creating load context for runtime %p",
-                  runtime);
-        context = g_object_new(GJS_TYPE_CONTEXT,
-                               "runtime", runtime,
-                               "is-load-context", TRUE,
-                               NULL);
-        gjs_runtime_set_data(runtime,
-                             "gjs-load-context",
-                             context,
-                             g_object_unref);
-    }
-
-    return (JSContext*)gjs_context_get_native_context(context);
-}
-
-JSContext*
-gjs_runtime_peek_load_context(JSRuntime *runtime)
-{
-    GjsContext *context;
-
-    context = gjs_runtime_get_data(runtime, "gjs-load-context");
-    if (context == NULL) {
-        return NULL;
-    } else {
-        return (JSContext*)gjs_context_get_native_context(context);
-    }
-}
-
-void
-gjs_runtime_clear_load_context(JSRuntime *runtime)
-{
-    gjs_debug(GJS_DEBUG_CONTEXT, "Clearing load context");
-    gjs_runtime_set_data(runtime,
-                         "gjs-load-context",
-                         NULL,
-                         NULL);
-    gjs_debug(GJS_DEBUG_CONTEXT, "Load context cleared");
-}
-
-/* The call context exists because when we call a closure, the scope
- * chain on the context is set to the original scope chain of the
- * closure. We want to avoid using any existing context (especially
- * the load context) because the closure "messes up" the scope chain
- * on the context.
+/**
+ * gjs_runtime_init:
+ * @runtime: a #JSRuntime
  *
- * Unlike the load context, which is expected to be an eternal
- * singleton, we only cache the call context for efficiency. It would
- * be just as workable to recreate it for each call.
+ * Initializes a #JSRuntime for use with GJS
+ *
+ * This should only be called by GJS, not by applications.
  */
-JSContext*
-gjs_runtime_get_call_context(JSRuntime *runtime)
-{
-    GjsContext *context;
-
-    context = gjs_runtime_get_data(runtime, "gjs-call-context");
-    if (context == NULL) {
-        gjs_debug(GJS_DEBUG_CONTEXT,
-                  "Creating call context for runtime %p",
-                  runtime);
-        context = g_object_new(GJS_TYPE_CONTEXT,
-                               "runtime", runtime,
-                               NULL);
-        gjs_runtime_set_data(runtime,
-                             "gjs-call-context",
-                             context,
-                             g_object_unref);
-    }
-
-    return (JSContext*)gjs_context_get_native_context(context);
-}
-
-static JSContext*
-gjs_runtime_peek_call_context(JSRuntime *runtime)
-{
-    GjsContext *context;
-
-    context = gjs_runtime_get_data(runtime, "gjs-call-context");
-    if (context == NULL) {
-        return NULL;
-    } else {
-        return (JSContext*)gjs_context_get_native_context(context);
-    }
-}
-
 void
-gjs_runtime_clear_call_context(JSRuntime *runtime)
+gjs_runtime_init(JSRuntime *runtime)
 {
-    gjs_debug(GJS_DEBUG_CONTEXT, "Clearing call context");
-    gjs_runtime_set_data(runtime,
-                         "gjs-call-context",
-                         NULL,
-                         NULL);
-    gjs_debug(GJS_DEBUG_CONTEXT, "Call context cleared");
+    RuntimeData *rd;
+
+    /* If we went back to supporting foreign contexts, we couldn't use
+     * JS_SetRuntimePrivate() because the runtime's owner might
+     * already be using it. A simple solution would be to just store
+     * the runtime data in a global variable - multiple copies of GJS
+     * in the same process at the same time have issues anyways
+     * because of limitations of GObject toggle references - if two
+     * separate entities toggle reference an object it will leak.
+     */
+    if (JS_GetRuntimePrivate(runtime) != NULL)
+        gjs_fatal("JSRuntime already initialized or private data in use by someone else");
+
+    rd = g_slice_new0(RuntimeData);
+    rd->dynamic_classes = g_hash_table_new(g_direct_hash, g_direct_equal);
+    JS_SetRuntimePrivate(runtime, rd);
 }
 
-static void
-runtime_data_destroy_notify(void *data)
+/**
+ * gjs_runtime_destroy:
+ * @runtime: a #JSRuntime
+ *
+ * Calls JS_DestroyRuntime() on runtime and frees data allocated by
+ * gjs_runtime_init(); these are unified into a single call because we
+ * need to order things so that the allocated data is cleaned up
+ * after JS_DestroyRuntime(). We might have finalizers run by
+ * JS_DestroyRuntime() that rely on the information stored in the data,
+ * such as the dynamic class structs.
+ *
+ * This should only be called by GJS, not by applications.
+ */
+void
+gjs_runtime_destroy(JSRuntime *runtime)
 {
-    RuntimeData *rd = data;
+    RuntimeData *rd;
     void *key;
     void *value;
 
+    rd = JS_GetRuntimePrivate(runtime);
+    if (rd->context_stack != NULL || rd->current_frame.depth != 0)
+        gjs_fatal("gjs_runtime_destroy() called during gjs_push_context()");
+
+    gjs_debug(GJS_DEBUG_CONTEXT,
+              "Destroying JS runtime");
+
+    JS_DestroyRuntime(runtime);
+
+    gjs_debug(GJS_DEBUG_CONTEXT,
+              "Destroying any remaining dataset items on runtime");
+
     while (gjs_g_hash_table_remove_one(rd->dynamic_classes, &key, &value)) {
         JSClass *clasp = value;
 
@@ -235,13 +350,9 @@ get_data_from_runtime(JSRuntime *runtime)
 {
     RuntimeData *rd;
 
-    rd = gjs_runtime_get_data(runtime, "gjs-api-util-data");
-    if (rd == NULL) {
-        rd = g_slice_new0(RuntimeData);
-        rd->dynamic_classes = g_hash_table_new(g_direct_hash, g_direct_equal);
-        gjs_runtime_set_data(runtime, "gjs-api-util-data",
-                             rd, runtime_data_destroy_notify);
-    }
+    rd = JS_GetRuntimePrivate(runtime);
+    if (G_UNLIKELY(rd == NULL))
+        gjs_fatal("JSRuntime not initialized for use with GJS");
 
     return rd;
 }
@@ -356,8 +467,8 @@ gjs_init_class_dynamic(JSContext      *context,
 {
     jsval value;
     char *private_name;
+    JSObject *global;
     JSObject *prototype;
-    JSContext *load_context;
 
     if (clasp->name != NULL) {
         g_warning("Dynamic class should not have a name in the JSClass struct");
@@ -366,14 +477,13 @@ gjs_init_class_dynamic(JSContext      *context,
 
     JS_BeginRequest(context);
 
-    /* We replace the passed-in context and global object with our
-     * runtime-global permanent load context. Otherwise, in a
-     * process with multiple contexts, we'd arbitrarily define
-     * the class in whatever global object initialized the
-     * class first, which is not desirable.
+    /* We use a special "fake" global object to store our constructors
+     * in for future use. Using the actual global object of the context would
+     * result in different contexts having different class definitions for
+     * the same GObject class; since the proxies are shared between all
+     * contexts, this would produce confusing results.
      */
-    load_context = gjs_runtime_get_load_context(JS_GetRuntime(context));
-    JS_BeginRequest(load_context);
+    global = gjs_get_import_global(context);
 
     /* JS_InitClass() wants to define the constructor in the global object, so
      * we give it a private and namespaced name... passing in the namespace
@@ -384,17 +494,17 @@ gjs_init_class_dynamic(JSContext      *context,
     private_name = g_strdup_printf("_private_%s_%s", ns_name, class_name);
 
     prototype = NULL;
-    if (gjs_object_get_property(load_context, JS_GetGlobalObject(load_context),
+    if (gjs_object_get_property(context, global,
                                 private_name, &value) &&
         JSVAL_IS_OBJECT(value)) {
         jsval proto_val;
 
         g_free(private_name); /* don't need it anymore */
 
-        if (!gjs_object_require_property(load_context, JSVAL_TO_OBJECT(value), NULL,
+        if (!gjs_object_require_property(context, JSVAL_TO_OBJECT(value), NULL,
                                          "prototype", &proto_val) ||
             !JSVAL_IS_OBJECT(proto_val)) {
-            gjs_throw(load_context, "prototype was not defined or not an object?");
+            gjs_throw(context, "prototype was not defined or not an object?");
             goto error;
         }
         prototype = JSVAL_TO_OBJECT(proto_val);
@@ -402,7 +512,7 @@ gjs_init_class_dynamic(JSContext      *context,
         DynamicJSClass *class_copy;
         RuntimeData *rd;
 
-        rd = get_data_from_context(load_context);
+        rd = get_data_from_context(context);
 
         class_copy = g_slice_new0(DynamicJSClass);
         class_copy->base = *clasp;
@@ -420,7 +530,7 @@ gjs_init_class_dynamic(JSContext      *context,
                   "Initializing dynamic class %s %p",
                   class_name, class_copy);
 
-        prototype = JS_InitClass(load_context, JS_GetGlobalObject(load_context),
+        prototype = JS_InitClass(context, global,
                                  parent_proto, &class_copy->base,
                                  constructor, nargs,
                                  ps, fs,
@@ -429,7 +539,7 @@ gjs_init_class_dynamic(JSContext      *context,
         /* Retrieve the property again so we can define it in
          * in_object
          */
-        if (!gjs_object_require_property(load_context, JS_GetGlobalObject(load_context), NULL,
+        if (!gjs_object_require_property(context, global, NULL,
                                          class_copy->base.name, &value))
             goto error;
     }
@@ -439,26 +549,17 @@ gjs_init_class_dynamic(JSContext      *context,
     /* Now manually define our constructor with a sane name, in the
      * namespace object.
      */
-    if (!JS_DefineProperty(load_context, in_object,
+    if (!JS_DefineProperty(context, in_object,
                            class_name,
                            value,
                            NULL, NULL,
                            GJS_MODULE_PROP_FLAGS))
         goto error;
 
-    JS_EndRequest(load_context);
     JS_EndRequest(context);
     return prototype;
 
  error:
-    /* Move the exception to the calling context from load context.
-     */
-    if (!gjs_move_exception(load_context, context)) {
-        /* set an exception since none was set */
-        gjs_throw(context, "No exception was set, but class initialize failed somehow");
-    }
-
-    JS_EndRequest(load_context);
     JS_EndRequest(context);
     return NULL;
 }
@@ -569,26 +670,24 @@ gjs_construct_object_dynamic(JSContext      *context,
 {
     RuntimeData *rd;
     JSClass *proto_class;
-    JSContext *load_context;
+    JSObject *global;
     JSObject *result;
 
     JS_BeginRequest(context);
 
-    /* We replace the passed-in context and global object with our
-     * runtime-global permanent load context. Otherwise, JS_ConstructObject
-     * can't find the constructor in whatever random global object is set
-     * on the passed-in context.
+    /* We use the "import global" rather than the global object for the current
+     * object so that we fine the constructors we stored there in
+     * js_init_class_dynamic.
      */
-    load_context = gjs_runtime_get_load_context(JS_GetRuntime(context));
-    JS_BeginRequest(load_context);
+    global = gjs_get_import_global(context);
 
-    proto_class = JS_GET_CLASS(load_context, proto);
+    proto_class = JS_GET_CLASS(context, proto);
 
-    rd = get_data_from_context(load_context);
+    rd = get_data_from_context(context);
 
     /* Check that it's safe to cast to DynamicJSClass */
     if (g_hash_table_lookup(rd->dynamic_classes, proto_class) == NULL) {
-        gjs_throw(load_context, "Prototype is not for a dynamically-registered class");
+        gjs_throw(context, "Prototype is not for a dynamically-registered class");
         goto error;
     }
 
@@ -596,27 +695,24 @@ gjs_construct_object_dynamic(JSContext      *context,
                         "Constructing instance of dynamic class %s %p from proto %p",
                         proto_class->name, proto_class, proto);
 
+    /* Passing in the import global as 'parent' results in it being the global object
+     * used for looking up the constructor for the object. It also results in
+     * it being stored as the parent object of the newly constructed object.
+     * (Not necessarily sensible, but for something like creating the proxy object
+     * for a GObject more sensible than using the global object of the current context.)
+     */
     if (argc > 0)
-        result = JS_ConstructObjectWithArguments(load_context, proto_class, proto, NULL, argc, argv);
+        result = JS_ConstructObjectWithArguments(context, proto_class, proto, global, argc, argv);
     else
-        result = JS_ConstructObject(load_context, proto_class, proto, NULL);
+        result = JS_ConstructObject(context, proto_class, proto, global);
 
     if (!result)
         goto error;
 
-    JS_EndRequest(load_context);
     JS_EndRequest(context);
     return result;
 
  error:
-    /* Move the exception to the calling context from load context.
-     */
-    if (!gjs_move_exception(load_context, context)) {
-        /* set an exception since none was set */
-        gjs_throw(context, "No exception was set, but object construction failed somehow");
-    }
-
-    JS_EndRequest(load_context);
     JS_EndRequest(context);
     return NULL;
 }
@@ -723,7 +819,7 @@ gjs_log_object_props(JSContext      *context,
      * that could get collected as we go through this process. So
      * create a local root scope.
      */
-    JS_EnterLocalRootScope(context);
+    (void)JS_EnterLocalRootScope(context);
 
     props_iter = JS_NewPropertyIterator(context, obj);
     if (props_iter == NULL) {
@@ -766,8 +862,6 @@ void
 gjs_explain_scope(JSContext  *context,
                   const char *title)
 {
-    JSContext *load_context;
-    JSContext *call_context;
     JSObject *global;
     JSObject *parent;
     GString *chain;
@@ -776,20 +870,13 @@ gjs_explain_scope(JSContext  *context,
               "=== %s ===",
               title);
 
-    load_context = gjs_runtime_peek_load_context(JS_GetRuntime(context));
-    call_context = gjs_runtime_peek_call_context(JS_GetRuntime(context));
-
     JS_BeginRequest(context);
-    JS_BeginRequest(load_context);
-    JS_BeginRequest(call_context);
 
-    JS_EnterLocalRootScope(context);
+    (void)JS_EnterLocalRootScope(context);
 
     gjs_debug(GJS_DEBUG_SCOPE,
               "  Context: %p %s",
               context,
-              context == load_context ? "(LOAD CONTEXT)" :
-              context == call_context ? "(CALL CONTEXT)" :
               "");
 
     global = JS_GetGlobalObject(context);
@@ -817,8 +904,6 @@ gjs_explain_scope(JSContext  *context,
 
     JS_LeaveLocalRootScope(context);
 
-    JS_EndRequest(call_context);
-    JS_EndRequest(load_context);
     JS_EndRequest(context);
 }
 
@@ -1029,18 +1114,12 @@ gjs_call_function_value(JSContext      *context,
                         jsval          *rval)
 {
     JSBool result;
-    JSContext *call_context;
 
     JS_BeginRequest(context);
 
-    call_context = gjs_runtime_get_call_context(JS_GetRuntime(context));
-    JS_BeginRequest(call_context);
-
-    result = JS_CallFunctionValue(call_context, obj, fval,
+    result = JS_CallFunctionValue(context, obj, fval,
                                   argc, argv, rval);
-    gjs_move_exception(call_context, context);
 
-    JS_EndRequest(call_context);
     JS_EndRequest(context);
     return result;
 }
