@@ -22,7 +22,8 @@
  */
 
 #include "mainloop.h"
-#include <gjs/gjs.h>
+#include <gjs/gjs-module.h>
+#include <gjs/compat.h>
 
 #include "../gi/closure.h"
 
@@ -103,7 +104,10 @@ gjs_main_loop_run(JSContext *context,
               context);
     g_free(cancel_id);
 
+    gjs_runtime_push_context(JS_GetRuntime(context), context);
     g_main_loop_run(main_loop);
+    gjs_runtime_pop_context(JS_GetRuntime(context));
+
     g_main_loop_unref(main_loop);
     return JS_TRUE;
 }
@@ -114,19 +118,21 @@ closure_source_func(void *data)
     jsval retval;
     GClosure *closure;
     JSBool bool_val;
+    JSRuntime *runtime;
     JSContext *context;
 
     closure = data;
 
-    context = gjs_closure_get_context(closure);
-    if (context == NULL) {
-        /* closure is invalid now */
+    if (!gjs_closure_is_valid(closure))
         return FALSE;
-    }
+
+    runtime = gjs_closure_get_runtime(closure);
+    context = gjs_runtime_get_current_context(runtime);
+
     JS_BeginRequest(context);
 
     retval = JSVAL_VOID;
-    JS_AddRoot(context, &retval);
+    JS_AddValueRoot(context, &retval);
 
     gjs_closure_invoke(closure,
                           0, NULL,
@@ -140,7 +146,7 @@ closure_source_func(void *data)
                            retval, &bool_val))
         bool_val = FALSE;
 
-    JS_RemoveRoot(context, &retval);
+    JS_RemoveValueRoot(context, &retval);
 
     JS_EndRequest(context);
     return bool_val;
